@@ -1,13 +1,13 @@
 # Booking.pm -- Base class for Bookings.
-# RCS Info        : $Id: Booking.pm,v 1.4 2006/01/22 16:32:30 jv Exp $
+# RCS Info        : $Id: Booking.pm,v 1.7 2006/03/05 20:54:38 jv Exp $
 # Author          : Johan Vromans
 # Created On      : Sat Oct 15 23:36:51 2005
 # Last Modified By: Johan Vromans
-# Last Modified On: Sun Jan 22 15:14:51 2006
-# Update Count    : 33
+# Last Modified On: Sun Mar  5 17:58:54 2006
+# Update Count    : 41
 # Status          : Unknown, Use with caution!
 
-my $RCS_Id = '$Id: Booking.pm,v 1.4 2006/01/22 16:32:30 jv Exp $ ';
+my $RCS_Id = '$Id: Booking.pm,v 1.7 2006/03/05 20:54:38 jv Exp $ ';
 
 package main;
 
@@ -22,6 +22,7 @@ use strict;
 use warnings;
 
 use EB;
+use EB::Finance;
 
 sub new {
     my ($class) = @_;
@@ -98,4 +99,63 @@ sub in_bky {
     1;
 }
 
+sub amount_with_btw {
+    my ($self, $amt, $btw_spec) = @_;
+    if ( $amt =~ /^(.+)\@(.+)$/ ) {
+	$amt = $1;
+	$btw_spec = $2;
+    }
+    return (amount($amt), $btw_spec);
+}
+
+sub parse_btw_spec {
+    my ($self, $spec, $btw_id, $kstomz) = @_;
+    return (0, undef) unless defined($spec);
+    $spec = lc($spec);
+
+    # Quickie for G/N.
+    if ( $spec =~ /^([gn])$/ ) {
+	return (0, undef);
+    }
+    # Strip off trailing K|O.
+    elsif ( $spec =~ /^(.*)([ko])(.*)$/ ) {
+	$kstomz = $2 eq 'k';
+	$spec = $1.$3;
+    }
+    elsif ( $spec =~ /^(.*)([iv])(.*)$/ ) {
+	$kstomz = $2 eq 'i';
+	$spec = $1.$3;
+	warn("!".__x("BTW specificatie {spec}: Gebruik K of O in plaats van I of V",
+		     spec => $_[0])."\n");
+    }
+
+    # Examine rest. Numeric -> BTW id.
+    if ( $spec =~ /^(\d+)$/ ) {
+	$btw_id = $1;
+    }
+    # H L H- L- H+ L+
+    elsif ( $spec =~ /^([hl])([-+])?$/ ) {
+	$btw_id = $1;
+	my $excl;
+	$excl = $2 eq '-' if defined $2;
+	$btw_id = $dbh->do("SELECT btw_id FROM BTWTabel".
+			   " WHERE btw_tariefgroep = ?".
+			   " AND ".($excl?"NOT ":"")."btw_incl",
+			   $btw_id eq "h" ? BTWTARIEF_HOOG : BTWTARIEF_LAAG)->[0];
+    }
+    # + -
+    elsif ( $spec =~ /^([-+])$/ && $btw_id ) {
+	$btw_id = $dbh->do("SELECT btw_id FROM BTWTabel".
+			   " WHERE btw_tariefgroep =".
+			   " ( SELECT btw_tariefgroep FROM BTWTabel".
+			   " WHERE btw_id = ? )".
+			   " AND ".($1 eq '-'?"NOT ":"")."btw_incl",
+			   $btw_id)->[0];
+    }
+    elsif ( $spec ne '' ) {
+	return;
+    }
+    ($btw_id, $kstomz);
+}
+
 1;
