@@ -1,5 +1,5 @@
 #!/usr/bin/perl -w
-my $RCS_Id = '$Id: BKM.pm,v 1.41 2006/02/07 11:42:56 jv Exp $ ';
+my $RCS_Id = '$Id: BKM.pm,v 1.47 2006/03/05 20:57:29 jv Exp $ ';
 
 package main;
 
@@ -13,8 +13,8 @@ package EB::Booking::BKM;
 # Author          : Johan Vromans
 # Created On      : Thu Jul  7 14:50:41 2005
 # Last Modified By: Johan Vromans
-# Last Modified On: Mon Feb  6 12:52:02 2006
-# Update Count    : 336
+# Last Modified On: Sun Mar  5 20:47:36 2006
+# Update Count    : 367
 # Status          : Unknown, Use with caution!
 
 ################ Common stuff ################
@@ -43,6 +43,7 @@ sub perform {
     my $dagboek_type = $opts->{dagboek_type};
     my $totaal = $opts->{totaal};
     my $saldo = $opts->{saldo};
+    my $does_btw = $dbh->does_btw;
 
     if ( defined($totaal) ) {
 	my $t = amount($totaal);
@@ -70,7 +71,7 @@ sub perform {
     else {
 	return "?".__x("Onherkenbare datum: {date}",
 		       date => $args->[0])."\n"
-	  if ($args->[0]||"") =~ /^[[:digit:]]/;
+	  if ($args->[0]||"") =~ /^[[:digit:]]+-/;
 	$date = iso8601date();
     }
 
@@ -108,7 +109,7 @@ sub perform {
 	    if ( $dd ) {
 		shift(@$args);
 		return unless $self->in_bky($dd, $begin, $end);
-		if ( $dbh->adm("btwbegin") && $dd lt $dbh->adm("btwbegin") ) {
+		if ( $does_btw && $dbh->adm("btwbegin") && $dd lt $dbh->adm("btwbegin") ) {
 		    warn("?"._T("De boekingsdatum valt in de periode waarover al BTW aangifte is gedaan")."\n");
 		    return;
 		}
@@ -116,7 +117,7 @@ sub perform {
 	    else {
 		return "?".__x("Onherkenbare datum: {date}",
 			       date => $args->[0])."\n"
-		  if ($args->[0]||"") =~ /^[[:digit:]]/;
+		  if ($args->[0]||"") =~ /^[[:digit:]]+-/;
 		$dd = $date;
 	    }
 	    return "?"._T("Deze opdracht is onvolledig. Gebruik de \"help\" opdracht voor meer aanwijzingen.")."\n"
@@ -129,22 +130,18 @@ sub perform {
 		warn(" "._T("boekstuk").": std $t $amt $acct\n");
 	    }
 
-	    my $dc = "acc_debcrd";
-	    my $explicit_dc;
-	    if ( $acct =~ /^(\d+)([cd])/i ) {
-#		$acct = $1;
-#		$explicit_dc = $dc = lc($2) eq 'd' ? 1 : 0;
-		warn("?"._T("De \"D\" of \"C\" toevoeging aan het rekeningnummer is hier niet toegestaan")."\n");
+	    if  ( $acct !~ /^\d+$/ ) {
+		if ( $acct =~ /^(\d+)([cd])/i ) {
+		    warn("?"._T("De \"D\" of \"C\" toevoeging aan het rekeningnummer is hier niet toegestaan")."\n");
+		}
+		else {
+		    warn("?".__x("Ongeldig grootboekrekeningnummer: {acct}", acct => $acct )."\n");
+		}
 		$fail++;
 		next;
 	    }
-	    elsif  ( $acct !~ /^\d+$/ ) {
-		warn("?".__x("Ongeldig grootboekrekeningnummer: {acct}", acct => $acct )."\n");
-		$fail++;
-		next;
-	    }
-	    $dc = 1;		# ####
-	    my $rr = $dbh->do("SELECT acc_desc,acc_balres,$dc,acc_kstomz,acc_btw".
+
+	    my $rr = $dbh->do("SELECT acc_desc,acc_balres,acc_kstomz,acc_btw".
 			      " FROM Accounts".
 			      " WHERE acc_id = ?", $acct);
 	    unless ( $rr ) {
@@ -153,18 +150,21 @@ sub perform {
 		$fail++;
 		next;
 	    }
-	    my ($adesc, $balres, $debcrd, $kstomz, $btw_id) = @$rr;
+	    my ($adesc, $balres, $kstomz, $btw_id) = @$rr;
 
 	    if ( $balres && $dagboek_type != DBKTYPE_MEMORIAAL ) {
 		warn("!".__x("Grootboekrekening {acct} ({desc}) is een balansrekening",
 			     acct => $acct, desc => $adesc)."\n") if 0;
-		#$dbh->rollback;
-		#return;
+	    }
+	    if ( $btw_id && !$does_btw ) {
+		croak("INTERNAL ERROR: ".
+		      __x("Grootboekrekening {acct} heeft BTW in een BTW-vrije administratie",
+			  acct => $acct));
 	    }
 
 	    my $bid;
 	    my $oamt = $amt;
-	    ($amt, $bid) = amount($amt, undef);
+	    ($amt, $bid) = $does_btw ? $self->amount_with_btw($amt, undef) : amount($amt);
 	    unless ( defined($amt) ) {
 		warn("?".__x("Ongeldig bedrag: {amt}", amt => $oamt)."\n");
 		$fail++;
@@ -172,59 +172,30 @@ sub perform {
 	    }
 	    $btw_id = 0, undef($bid) if defined($bid) && !$bid; # override: @0
 
-	    # If there's BTW associated, it must be explicitly confirmed.
+	    # For memorials, if there's BTW associated, it must be explicitly confirmed.
 	    if ( $btw_id && !defined($bid) && $dagboek_type == DBKTYPE_MEMORIAAL ) {
-		warn("?".__x("Boekingen met BTW zijn niet mogelijk in een {dbk}.".
-			     " De BTW is op nul gesteld.",
-			     dbk => $dagboek_type == DBKTYPE_BANK ? "bankboek" :
-			     $dagboek_type == DBKTYPE_KAS ? "kasboek" :
-			     "memoriaal")."\n");
+		warn("?"._T("Boekingen met BTW zijn niet mogelijk in een memoriaal.".
+			    " De BTW is op nul gesteld.")."\n");
 		$btw_id = 0;
 	    }
+
 	    my $btw_acc;
 	    if ( defined($bid) ) {
-		if ( $bid =~ /^[hl]|[hl][iv]|[iv][hl]$/i ) {
-		    $bid = lc($bid);
-		    my $t = $bid =~ /h/ ? "h" : "l";
-		    if ( $bid =~ /([iv])/ ) {
-			$t = $1.$t;
-		    }
-		    else {
-			$t = $amt < 0 ? "i$t" : "v$t";
-		    }
-		    $btw_acc = $dbh->std_acc("btw_$t");
-		    $btw_id = $dbh->do("SELECT btw_id".
-				       " FROM BTWTabel".
-				       " WHERE btw_tariefgroep = ?".
-				       " AND btw_incl",
-				       $bid =~ /h/ ? BTWTARIEF_HOOG : BTWTARIEF_LAAG)->[0];
+
+		($btw_id, $kstomz) = $self->parse_btw_spec($bid, $btw_id, $kstomz);
+		unless ( defined($btw_id) ) {
+		    warn("?".__x("Ongeldige BTW-specificatie: {spec}", spec => $bid)."\n");
+		    $fail++;
+		    next;
 		}
-		elsif ( $bid =~ /^\d+|\d+[iv]|[iv]\d+$/i ) {
-		    my $t = $btw_id = $1 if $bid =~ /(\d+)/;
-		    my $group = $dbh->lookup($t, qw(BTWTabel btw_id btw_tariefgroep));
-		    unless ( defined $group ) {
-			warn("?".__x("Ongeldige BTW codering: {cod}",
-				     cod => '@'.$bid)."\n");
-			$fail++;
-			next;
-		    }
-		    if ( $bid =~ /([iv])/ ) {
-			$t = $1;
-		    }
-		    else {
-			$t = $kstomz ? "i" : "v";
-		    }
-		    $t .= $group == BTWTARIEF_HOOG ? "h" : "l";
-		    $btw_acc = $dbh->std_acc("btw_$t");
-		}
-		else {
-		    warn("?".__x("Ongeldige BTW codering: {cod}",
-				 cod => '@'.$bid)."\n");
+
+		if ( !defined($kstomz) && $btw_id ) {
+		    warn("?"._T("BTW toepassen is niet mogelijk op een neutrale rekening")."\n");
 		    $fail++;
 		    next;
 		}
 	    }
-	    elsif ( $btw_id ) {
+	    if ( $btw_id ) {
 		my $tg = $dbh->lookup($btw_id, qw(BTWTabel btw_id btw_tariefgroep));
 		croak("INTERNAL ERROR: btw code $btw_id heeft tariefgroep $tg")
 		  unless $tg;
@@ -241,7 +212,7 @@ sub perform {
 		  @{EB::Finance::norm_btw($bsr_amount, $btw_id)};
 		$amt = $bsr_amount - $btw;
 	    }
-	    $orig_amount = -$orig_amount;# unless $debcrd;
+	    $orig_amount = -$orig_amount;
 
 	    $dbh->sql_insert("Boekstukregels",
 			     [qw(bsr_nr bsr_date bsr_bsk_id bsr_desc bsr_amount
@@ -249,18 +220,18 @@ sub perform {
 				 bsr_acc_id bsr_rel_code)],
 			     $nr++, $dd, $bsk_id, $desc, $orig_amount,
 			     $btw_id, $btw_acc,
-			     BTWKLASSE(1, BTWTYPE_NORMAAL, $balres ? 1 : $kstomz),
+			     BTWKLASSE($does_btw ? defined($kstomz) : 0, BTWTYPE_NORMAAL, $kstomz||0),
 			     0, $acct, undef);
 
-	    warn("update $acct with ".numfmt(-$amt)."\n") if $trace_updates;
-	    $dbh->upd_account($acct, -$amt);
+#	    warn("update $acct with ".numfmt(-$amt)."\n") if $trace_updates;
+#	    $dbh->upd_account($acct, -$amt);
 	    $tot += $amt;
 
 	    if ( $btw ) {
-		my $btw_acct =
-		  $dbh->lookup($acct, qw(Accounts acc_id acc_debcrd)) ? $btw_ink : $btw_verk;
-		warn("update $btw_acct with ".numfmt(-$btw)."\n") if $trace_updates;
-		$dbh->upd_account($btw_acct, -$btw);
+#		my $btw_acct =
+#		  $dbh->lookup($acct, qw(Accounts acc_id acc_debcrd)) ? $btw_ink : $btw_verk;
+#		warn("update $btw_acct with ".numfmt(-$btw)."\n") if $trace_updates;
+#		$dbh->upd_account($btw_acct, -$btw);
 		$tot += $btw;
 	    }
 
@@ -274,7 +245,7 @@ sub perform {
 	    if ( $dd ) {
 		shift(@$args);
 		return unless $self->in_bky($dd, $begin, $end);
-		if ( $dbh->adm("btwbegin") && $dd lt $dbh->adm("btwbegin") ) {
+		if ( $does_btw && $dbh->adm("btwbegin") && $dd lt $dbh->adm("btwbegin") ) {
 		    warn("?"._T("De boekingsdatum valt in de periode waarover al BTW aangifte is gedaan")."\n");
 		    return;
 		}
@@ -282,7 +253,7 @@ sub perform {
 	    else {
 		return "?".__x("Onherkenbare datum: {date}",
 			       date => $args->[0])."\n"
-		  if ($args->[0]||"") =~ /^[[:digit:]]/;
+		  if ($args->[0]||"") =~ /^[[:digit:]]+-/;
 		$dd = $date;
 	    }
 	    return "?"._T("Deze opdracht is onvolledig. Gebruik de \"help\" opdracht voor meer aanwijzingen.")."\n"
@@ -319,8 +290,6 @@ sub perform {
 		$rr = $dbh->do($sql, @sql_args);
 		unless ( defined($rr) ) {
 		    warn("?"._T("Geen bijbehorende open post gevonden")."\n");
-		    #warn("DEBUG: SQL: $sql\n");
-		    #warn("DEBUG: args: @sql_args\n");
 		    $fail++;
 		    next;
 		}
@@ -357,8 +326,6 @@ sub perform {
 		$rr = $dbh->do($sql, @sql_args);
 		unless ( defined($rr) ) {
 		    warn("?"._T("Geen bijbehorende open post gevonden")."\n");
-		    #warn("DEBUG: SQL: $sql\n");
-		    #warn("DEBUG: args: @sql_args\n");
 		    $fail++;
 		    next;
 		}
@@ -380,8 +347,8 @@ sub perform {
 			   " WHERE bsk_id = ?",
 			   $amt, $bskid);
 
-	    warn("update $acct with ".numfmt(-$amt)."\n") if $trace_updates;
-	    $dbh->upd_account($acct, -$amt);
+#	    warn("update $acct with ".numfmt(-$amt)."\n") if $trace_updates;
+#	    $dbh->upd_account($acct, -$amt);
 	    $tot += $amt;
 	}
 	else {
