@@ -1,5 +1,5 @@
 #!/usr/bin/perl -w
-my $RCS_Id = '$Id: IV.pm,v 1.35 2006/02/06 11:56:31 jv Exp $ ';
+my $RCS_Id = '$Id: IV.pm,v 1.40 2006/03/05 20:57:28 jv Exp $ ';
 
 package main;
 
@@ -13,8 +13,8 @@ package EB::Booking::IV;
 # Author          : Johan Vromans
 # Created On      : Thu Jul  7 14:50:41 2005
 # Last Modified By: Johan Vromans
-# Last Modified On: Mon Feb  6 12:53:23 2006
-# Update Count    : 226
+# Last Modified On: Sun Mar  5 20:47:13 2006
+# Update Count    : 272
 # Status          : Unknown, Use with caution!
 
 ################ Common stuff ################
@@ -50,6 +50,7 @@ sub perform {
 
     my $iv = $dagboek_type == DBKTYPE_INKOOP;
     my $totaal = $opts->{totaal};
+    my $does_btw = $dbh->does_btw;
 
     my $bky = $self->{bky} ||= $opts->{boekjaar} || $dbh->adm("bky");
 
@@ -70,7 +71,7 @@ sub perform {
     else {
 	return "?".__x("Onherkenbare datum: {date}",
 		       date => $args->[0])."\n"
-	  if ($args->[0]||"") =~ /^[[:digit:]]/;
+	  if ($args->[0]||"") =~ /^[[:digit:]]+-/;
 	$date = iso8601date();
     }
 
@@ -79,7 +80,7 @@ sub perform {
 
     return unless $self->in_bky($date, $begin, $end);
 
-    if ( $dbh->adm("btwbegin") && $date lt $dbh->adm("btwbegin") ) {
+    if ( $does_btw && $dbh->adm("btwbegin") && $date lt $dbh->adm("btwbegin") ) {
 	warn("?"._T("De boekingsdatum valt in de periode waarover al BTW aangifte is gedaan")."\n");
 	return;
     }
@@ -139,8 +140,8 @@ sub perform {
 	}
     }
 
-    my ($rel_acc_id, $sbtw);
-    ($debcode, $rel_acc_id, $sbtw) = @$rr;
+    my ($rel_acc_id, $rel_btw);
+    ($debcode, $rel_acc_id, $rel_btw) = @$rr;
 
     my $nr = 1;
     my $bsk_id;
@@ -158,19 +159,15 @@ sub perform {
 	    warn(" "._T("boekstuk").": $t $amt $acct\n");
 	}
 
-	my $dc = "acc_debcrd";
-	my $explicit_dc;
-	if ( $acct =~ /^(\d*)([cd])/i ) {
-	    warn("?"._T("De \"D\" of \"C\" toevoeging aan het rekeningnummer is hier niet toegestaan")."\n");
-	    return;
-#	    $acct = $1 || $rel_acc_id;
-#	    $explicit_dc = $dc = lc($2) eq 'd' ? 1 : 0;
-	}
-	elsif  ( $acct !~ /^\d+$/ ) {
+	if  ( $acct !~ /^\d+$/ ) {
+	    if ( $acct =~ /^(\d*)([cd])/i ) {
+		warn("?"._T("De \"D\" of \"C\" toevoeging aan het rekeningnummer is hier niet toegestaan")."\n");
+		return;
+	    }
 	    warn("?".__x("Ongeldig grootboekrekeningnummer: {acct}", acct => $acct )."\n");
 	    return;
 	}
-	my $rr = $dbh->do("SELECT acc_desc,acc_balres,acc_kstomz,$dc,acc_btw".
+	my $rr = $dbh->do("SELECT acc_desc,acc_balres,acc_kstomz,acc_debcrd,acc_btw".
 			  " FROM Accounts".
 			  " WHERE acc_id = ?", $acct);
 	unless ( $rr ) {
@@ -180,20 +177,16 @@ sub perform {
 	    return;
 	}
 	my ($adesc, $balres, $kstomz, $debcrd, $btw_id) = @$rr;
-
 	if ( $balres ) {
 	    warn("!".__x("Grootboekrekening {acct} ({desc}) is een balansrekening",
 			 acct => $acct, desc => $adesc)."\n") if 0;
 	    #$dbh->rollback;
 	    #return;
 	}
-	elsif ( $kstomz ? !$iv : $iv ) {
-	    warn("!".__x("Grootboekrekening {acct} ({desc}) is een {what}rekening",
-			 acct => $acct, desc => $adesc,
-			 what => $kstomz ? _T("kosten") : _T("omzet"),
-			)."\n");
-	    #$dbh->rollback;
-	    #return;
+	if ( $btw_id && !$does_btw ) {
+	    croak("INTERNAL ERROR: ".
+		  __x("Grootboekrekening {acct} heeft BTW in een BTW-vrije administratie",
+		      acct => $acct));
 	}
 
 	if ( $nr == 1 ) {
@@ -205,70 +198,77 @@ sub perform {
 	    $bsk_id = $dbh->get_sequence("boekstukken_bsk_id_seq", "noincr");
 	}
 
-	# btw_id    btw_acc
-	#   0         \N          zonder BTW, extra/verlegd
-	#   0         nnnn        zonder BTW
-	#   n         nnnn        normaal
-	#   n         \N          extra/verlegd
-
 	# Amount can override BTW id with @X postfix.
-	my $oamt = $amt;
-	($amt, $btw_id) = amount($amt, $btw_id);
-	unless ( defined($amt) ) {
-	    warn("?".__x("Ongeldig bedrag: {amt}", amt => $oamt)."\n");
+	my ($namt, $btw_spec) = $does_btw ? $self->amount_with_btw($amt, $btw_id) : amount($amt);
+	unless ( defined($namt) ) {
+	    warn("?".__x("Ongeldig bedrag: {amt}", amt => $amt)."\n");
 	    return;
 	}
 
-	$amt = -$amt unless $iv;
+	$amt = $iv ? $namt : -$namt;
 
-	my $btw_acc;
-	my $t = "btw_" . ($iv ? "i" : "v");
-	# Geen BTW voor non-EU.
-	if ( $btw_id && ($sbtw == BTWTYPE_NORMAAL || $sbtw == BTWTYPE_INTRA) ) {
-	    if ( $btw_id =~ /^[hl]$/i ) {
-		$t .= lc($btw_id);
-		$btw_id = $dbh->do("SELECT btw_id".
-				   " FROM BTWTabel".
-				   " WHERE btw_tariefgroep = ?".
-				   " AND btw_incl",
-				   lc($btw_id) eq 'h' ? BTWTARIEF_HOOG : BTWTARIEF_LAAG)->[0];
+	if ( $does_btw ) {
+	    ($btw_id, $kstomz) = $self->parse_btw_spec($btw_spec, $btw_id, $kstomz);
+	    unless ( defined($btw_id) ) {
+		warn("?".__x("Ongeldige BTW-specificatie: {spec}", spec => $btw_spec)."\n");
+		return;
 	    }
-	    else {
-		my $group = $dbh->lookup($btw_id, qw(BTWTabel btw_id btw_tariefgroep));
-		$t .= ($group == BTWTARIEF_HOOG ? "h" : "l");
-	    }
-	    $btw_acc = $dbh->std_acc($t);
 	}
 
+	# Bepalen van de BTW.
+	# Voor neutrale boekingen (@N, of op een neutrale rekening) wordt geen BTW
+	# toegepast. Op _alle_ andere wel. De BTW kan echter nul zijn, of void.
+	# Het eerste wordt bewerkstelligd door $btw_id op 0 te zetten, het tweede
+	# door $btw_acc geen waarde te geven.
 	my $btwclass = 0;
-	# Inkoop: alle bedragen met BTW en intra/extra bedragen zijn van belang.
-	if ( $iv ) {
-	    $btwclass = BTWKLASSE(1, $sbtw, 1)
-	      if $btw_id || $sbtw == BTWTYPE_INTRA || $sbtw == BTWTYPE_EXTRA;
+	my $btw_acc;
+	if ( defined($kstomz) ) {
+	    # BTW toepassen.
+	    if ( $kstomz ? !$iv : $iv ) {
+		#warn("?".__x("U kunt geen {ko} boeken in een {iv} dagboek",
+		warn("!".__x("Pas op! U boekt {ko} in een {iv} dagboek",
+			     ko => $kstomz ? _T("kosten") : _T("omzet"),
+			     iv => $iv ? _T("inkoop") : _T("verkoop"),
+			    )."\n");
+		#return;
+	    }
+	    # Void BTW voor non-EU en verlegd.
+	    if ( $btw_id && ($rel_btw == BTWTYPE_NORMAAL || $rel_btw == BTWTYPE_INTRA) ) {
+		my $tg = $dbh->lookup($btw_id, qw(BTWTabel btw_id btw_tariefgroep));
+		unless ( defined($tg) ) {
+		    warn("?".__x("Onbekende BTW-code: {code}", code => $btw_id)."\n");
+		    return;
+		}
+		my $t = "btw_" . ($iv ? "i" : "v");
+		$t .= $tg == BTWTARIEF_HOOG ? 'h' : 'l';
+		$btw_acc = $dbh->std_acc($t);
+	    }
 	}
-	# Verkoop: alle bedragen met BTW, en alle omzetten met 0%.
-	# Zo ook intra en extra.
-	else {
-	    $btwclass = BTWKLASSE(1, $sbtw, 0)
-	      if $btw_id || !$kstomz || $sbtw == BTWTYPE_INTRA || $sbtw == BTWTYPE_EXTRA;
+	elsif ( $btw_id ) {
+	    warn("?"._T("BTW toepassen is niet mogelijk op een neutrale rekening")."\n");
+	    return;
 	}
+	# ASSERT: $btw_id != 0 implies defined($kstomz).
+
 	$dbh->sql_insert("Boekstukregels",
 			 [qw(bsr_nr bsr_date bsr_bsk_id bsr_desc bsr_amount
 			     bsr_btw_id bsr_btw_acc bsr_btw_class bsr_type bsr_acc_id bsr_rel_code)],
 			 $nr++, $date, $bsk_id, $desc, $amt,
-			 $btw_id, $btw_acc, $btwclass, 0, $acct, $debcode);
+			 $btw_id, $btw_acc,
+			 BTWKLASSE($does_btw ? defined($kstomz) : 0, $rel_btw, defined($kstomz) ? $kstomz : $iv),
+			 0, $acct, $debcode);
     }
 
     my $ret = EB::Finance::journalise($bsk_id);
-    $rr = [ @$ret ];
-    shift(@$rr);
-    $rr = [ sort { $a->[5] <=> $b->[5] } @$rr ];
-    foreach my $r ( @$rr ) {
-	my (undef, undef, undef, undef, $nr, $ac, $amt) = @$r;
-	next unless $nr;
-	warn("update $ac with ".numfmt($amt)."\n") if $trace_updates;
-	$dbh->upd_account($ac, $amt);
-    }
+#    $rr = [ @$ret ];
+#    shift(@$rr);
+#    $rr = [ sort { $a->[5] <=> $b->[5] } @$rr ];
+#    foreach my $r ( @$rr ) {
+#	my (undef, undef, undef, undef, $nr, $ac, $amt) = @$r;
+#	next unless $nr;
+#	warn("update $ac with ".numfmt($amt)."\n") if $trace_updates;
+#	$dbh->upd_account($ac, $amt);
+#    }
     my $tot = $ret->[$#{$ret}]->[6];
     $dbh->sql_exec("UPDATE Boekstukken SET bsk_amount = ?, bsk_open = ? WHERE bsk_id = ?",
 		   $tot, $tot, $bsk_id)->finish;
