@@ -1,11 +1,11 @@
 #!/usr/bin/perl -w
-my $RCS_Id = '$Id: DB.pm,v 1.37 2006/02/07 11:46:28 jv Exp $ ';
+my $RCS_Id = '$Id: DB.pm,v 1.40 2006/03/05 20:55:17 jv Exp $ ';
 
 # Author          : Johan Vromans
 # Created On      : Sat May  7 09:18:15 2005
 # Last Modified By: Johan Vromans
-# Last Modified On: Tue Feb  7 12:28:53 2006
-# Update Count    : 309
+# Last Modified On: Sun Mar  5 21:37:37 2006
+# Update Count    : 326
 # Status          : Unknown, Use with caution!
 
 ################ Common stuff ################
@@ -99,7 +99,8 @@ sub check_db {
 	sprintf("%03d%03d", $min, $rev) eq sprintf("%03d%03d", SCM_MINVERSION, SCM_REVISION);
 
     # Verify koppelingen.
-    for ( $self->std_acc("deb") ) {
+    for ( $self->std_acc("deb", undef) ) {
+	next unless defined;
 	my $rr = $self->do("SELECT acc_debcrd, acc_balres FROM Accounts where acc_id = ?", $_);
 	$fail++, warn("?".__x("Geen grootboekrekening voor {dc} ({acct})",
 			      dc => _T("Debiteuren"), acct => $_)."\n")
@@ -110,7 +111,8 @@ sub check_db {
 	  unless $rr->[0] && $rr->[1];
     }
 
-    for ( $self->std_acc("crd") ) {
+    for ( $self->std_acc("crd", undef) ) {
+	next unless defined;
 	my $rr = $self->do("SELECT acc_debcrd, acc_balres FROM Accounts where acc_id = ?", $_);
 	$fail++, warn("?".__x("Geen grootboekrekening voor {dc} ({acct})",
 			      dc => _T("Crediteuren"), acct => $_)."\n")
@@ -121,7 +123,8 @@ sub check_db {
 	  if $rr->[0] || !$rr->[1];
     }
 
-    for ( $self->std_acc("btw_ok") ) {
+    for ( $self->std_acc("btw_ok", undef) ) {
+	next unless defined;
 	my $rr = $self->do("SELECT acc_balres FROM Accounts where acc_id = ?", $_);
 	$fail++, warn("?".__x("Geen grootboekrekening voor {dc} ({acct})",
 			      dc => _T("BTW betaald"), acct => $_)."\n")
@@ -141,7 +144,10 @@ sub check_db {
 	  unless $rr->[0];
     }
 
+=begin notanymore
+
     # Verify some grootboekrekeningen.
+    # We cannot do this anymore now we have free choice of accounts for ledgers.
     my $sth = $self->sql_exec("SELECT acc_id, acc_desc, dbk_id, dbk_type, dbk_desc FROM Dagboeken, Accounts".
 			      " WHERE dbk_acc_id = acc_id");
     while ( my $rr = $sth->fetchrow_arrayref ) {
@@ -158,6 +164,8 @@ sub check_db {
 	}
     }
 
+=cut
+
     die("?"._T("CONSISTENTIE-VERIFICATIE STANDAARDREKENINGEN MISLUKT")."\n") if $fail;
 
     # Create temp table for account mangling.
@@ -166,6 +174,7 @@ sub check_db {
     $dbh->commit;
 }
 
+#### UNUSED
 sub upd_account {
     my ($self, $acc, $amt) = @_;
     my $op = '+';		# perfectionism
@@ -306,15 +315,16 @@ sub dbver {
 my %std_acc;
 my @std_acc;
 sub std_acc {
-    my ($self, $name) = @_;
+    my ($self, $name, $def) = @_;
     if ( $name eq "" ) {
 	%std_acc = ();
 	@std_acc = ();
 	return;
     }
     $self->std_accs unless %std_acc;
-    $std_acc{lc($name)} || die("?".__x("Niet-bestaande standaardrekening: {std}",
-				       std => $name)."\n");
+    return $std_acc{lc($name)} if defined($std_acc{lc($name)});
+    return $def if @_ > 2;
+    die("?".__x("Niet-bestaande standaardrekening: {std}", std => $name)."\n");
 }
 
 sub std_accs {
@@ -365,6 +375,15 @@ sub adm_busy {
     $self->do("SELECT COUNT(*) FROM Journal")->[0];
 }
 
+sub does_btw {
+    my ($self) = @_;
+    $self->connectdb;
+    return defined($self->adm("btwbegin")) if $self->adm_open;
+    $self->do("SELECT COUNT(*)".
+	      " FROM BTWTabel".
+	      " WHERE btw_tariefgroep != 0")->[0];
+}
+
 ################ API calls for database backend ################
 
 my $dbpkg;
