@@ -1,10 +1,10 @@
 # Export.pm -- Export EekBoek administratie
-# RCS Info        : $Id: Export.pm,v 1.6 2006/02/02 11:36:07 jv Exp $
+# RCS Info        : $Id: Export.pm,v 1.11 2006/03/09 14:29:11 jv Exp $
 # Author          : Johan Vromans
 # Created On      : Mon Jan 16 20:47:38 2006
 # Last Modified By: Johan Vromans
-# Last Modified On: Tue Jan 31 21:22:07 2006
-# Update Count    : 123
+# Last Modified On: Thu Mar  9 15:27:33 2006
+# Update Count    : 135
 # Status          : Unknown, Use with caution!
 
 package main;
@@ -35,7 +35,7 @@ sub export {
 	$self->_write("$dir/schema.dat",  sub { $self->_schema(shift) });
 	$self->_write("$dir/relaties.eb", sub { print { shift } $self->_relaties });
 	$self->_write("$dir/opening.eb",  sub { print { shift } $self->_opening  });
-	$self->_write("$dir/mutaties.eb", sub { print { shift } $self->_mutaties });
+	$self->_write("$dir/mutaties.eb", sub { print { shift } $self->_mutaties($opts) });
 
     }
     else {
@@ -106,6 +106,8 @@ sub _relaties {
 sub _opening {
     my ($self) = @_;
 
+    require EB::Booking::Decode;
+
     my $sth;
 
     my $out = __x("# {what} voor administratie {adm}",
@@ -125,7 +127,7 @@ sub _opening {
     $out .= "adm_boekjaarcode " . _quote($dbh->lookup($begin, qw(Boekjaren bky_begin bky_code))) . "\n";
     $out .= "adm_btwperiode   " .
       (qw(geen jaar x x kwartaal x x x x x x x maand)[$dbh->adm("btwperiod")]).
-	"\n";
+	"\n" if $dbh->does_btw;
 
     $out .= "\n# " . _T("Openingsbalans") . "\n";
 
@@ -152,16 +154,16 @@ sub _opening {
 	    $out .= "\n# " . ($acc_debcrd ? _T("Debet") : _T("Credit")) . "\n";
 	}
 	$out .= sprintf("adm_balans %-5s %10s   # %s\n",
-			$acc_id, numfmt($acc_balance),
+			$acc_id, numfmt_plain($acc_balance),
 			$acc_desc);
 	$debcrd = $acc_debcrd;
     }
 
     die("?".__x("Internal error -- unbalance {arg1} <> {arg2}",
-		arg1 => numfmt($dt),
-		arg2 => numfmt($ct))."\n")
+		arg1 => numfmt_plain($dt),
+		arg2 => numfmt_plain($ct))."\n")
       unless $dt == $ct;
-    $out .= "\n# " .  _T("Totaal") . "\n" . "adm_balanstotaal " . numfmt($dt) . "\n";
+    $out .= "\n# " .  _T("Totaal") . "\n" . "adm_balanstotaal " . numfmt_plain($dt) . "\n";
 
 =begin wrong
 
@@ -186,7 +188,7 @@ sub _opening {
 		     "adm_relatie",
 		     $dbk_desc . ":" . $bsk_nr,
 		     $bsk_date, _quote($bsk_desc), _quote($bsr_rel_code),
-		     numfmt($bsk_amount)). "\n";
+		     numfmt_plain($bsk_amount)). "\n";
     }
 
 =cut
@@ -222,7 +224,7 @@ sub _opening {
 }
 
 sub _mutaties {
-    my ($self) = @_;
+    my ($self, $opts) = @_;
 
     my $out = __x("# {what} voor administratie {adm}",
 		  what => _T("Boekingen"), adm => $dbh->adm("name")) . "\n" .
@@ -254,6 +256,13 @@ sub _mutaties {
 	    }
 	    $sth->finish;
 	}
+	if ( $dbh->does_btw ) {
+	    my $bb = $dbh->adm("btwbegin");
+	    my $bkb = $dbh->lookup($bky, qw(Boekjaren bky_code bky_begin));
+	    my $bke = $dbh->lookup($bky, qw(Boekjaren bky_code bky_end));
+	    $bke = parse_date($bb, undef, -1) if $bb le $bke;
+	    $out .= "btwaangifte --periode=$bkb-$bke --definitief --noreport\n";
+	}
     };
 
     my $cur_bky = $bky[0];
@@ -284,10 +293,10 @@ sub _mutaties {
 	      ($bsk_id,
 	       { trail  => 1,
 		 d_boekjaar => $bky,
-		 single => 0,
-		 btw    => 0,
 		 bsknr  => 1,
-		 total  => 1,
+		 single => $opts->{single}   || 0,
+		 btw    => $opts->{explicit} || 0,
+		 total  => defined($opts->{totals}) ? $opts->{totals} : 1,
 		 debcrd => 0 }) . "\n";
 	}
 	$out .= "\n";
