@@ -1,5 +1,5 @@
 #!/usr/bin/perl -w
-my $RCS_Id = '$Id: Grootboek.pm,v 1.22 2006/01/22 16:42:11 jv Exp $ ';
+my $RCS_Id = '$Id: Grootboek.pm,v 1.23 2006/03/06 15:46:43 jv Exp $ ';
 
 package main;
 
@@ -13,8 +13,8 @@ package EB::Report::Grootboek;
 # Author          : Johan Vromans
 # Created On      : Wed Jul 27 11:58:52 2005
 # Last Modified By: Johan Vromans
-# Last Modified On: Fri Jan 20 21:59:31 2006
-# Update Count    : 240
+# Last Modified On: Mon Mar  6 11:04:32 2006
+# Update Count    : 241
 # Status          : Unknown, Use with caution!
 
 ################ Common stuff ################
@@ -47,8 +47,8 @@ sub perform {
       [ { name => "acct", title => _T("GrBk"),               width =>  5, align => ">" },
 	{ name => "desc", title => _T("Grootboek/Boekstuk"), width => 30,              },
 	{ name => "date", title => _T("Datum"),              width => 10, align => ">" },
-	{ name => "deb",  title => _T("Debet"),              width =>  9, align => ">" },
-	{ name => "crd",  title => _T("Credit"),             width =>  9, align => ">" },
+	{ name => "deb",  title => _T("Debet"),              width => $amount_width, align => ">" },
+	{ name => "crd",  title => _T("Credit"),             width => $amount_width, align => ">" },
 	{ name => "bsk",  title => _T("BoekstukNr"),         width => 14,              },
 	{ name => "rel",  title => _T("Relatie"),            width => 10,              },
       ];
