@@ -1,12 +1,12 @@
 #!/usr/bin/perl
 
-my $RCS_Id = '$Id: Shell.pm,v 1.60 2006/02/07 11:54:59 jv Exp $ ';
+my $RCS_Id = '$Id: Shell.pm,v 1.70 2006/03/10 12:51:24 jv Exp $ ';
 
 # Author          : Johan Vromans
 # Created On      : Thu Jul  7 15:53:48 2005
 # Last Modified By: Johan Vromans
-# Last Modified On: Tue Feb  7 12:54:55 2006
-# Update Count    : 740
+# Last Modified On: Thu Mar  9 13:07:05 2006
+# Update Count    : 773
 # Status          : Unknown, Use with caution!
 
 ################ Common stuff ################
@@ -32,7 +32,17 @@ sub new {
     _plug_cmds();
 
     # User defined stuff.
-    eval { require EB::Shell::Userdefs };
+    my $pkg = $cfg->val(qw(shell userdefs), undef);
+    if ( $pkg ) {
+	$pkg =~ s/::/\//g;
+	$pkg .= ".pm";
+	eval { require $pkg };
+	die($@) if $@;
+    }
+    else {
+	eval { require EB::Shell::Userdefs };
+	die($@) if $@ && $@ !~ /can't locate eb.shell.userdefs\.pm in \@inc/i;
+    }
 
     my $self = $class->SUPER::new($opts);
 
@@ -160,7 +170,6 @@ sub parseline {
 ################ Subroutines ################
 
 use EB;
-use EB::Tools::Opening;
 
 # Standard options for report generating backends.
 my @outopts;
@@ -168,6 +177,7 @@ INIT { @outopts = qw(html csv text output=s page=i) }
 
 # Plug in some commands dynamically.
 sub _plug_cmds {
+    my $does_btw = $dbh->does_btw;
     my $sth = $dbh->sql_exec("SELECT dbk_id,dbk_desc,dbk_type FROM Dagboeken");
     my $rr;
     while ( $rr = $sth->fetchrow_arrayref ) {
@@ -194,9 +204,11 @@ sub _plug_cmds {
     }
 
     # Opening (adm_...) commands.
+    require EB::Tools::Opening;
     foreach my $adm ( @{EB::Tools::Opening->commands} ) {
 	my $cmd = $adm;
 	$cmd =~ s/^set_//;
+	next if $cmd =~ /^btw/ && !$does_btw;
 	no strict 'refs';
 	*{"do_adm_$cmd"} = sub {
 	    (shift->{o} ||= EB::Tools::Opening->new)->$adm(@_);
@@ -209,6 +221,13 @@ sub _plug_cmds {
 	};
     }
 
+    # BTW aangifte.
+    if ( $does_btw ) {
+	no strict 'refs';
+	*{"do_btwaangifte"}   = \&_do_btwaangifte;
+	*{"help_btwaangifte"} = \&_help_btwaangifte;
+    }
+
     $dbk_pat = $dbk_i_pat.$dbk_v_pat.$dbk_bkm_pat;
     chop foreach ($dbk_pat, $dbk_i_pat, $dbk_v_pat, $dbk_bkm_pat);
 }
@@ -386,6 +405,7 @@ sub do_journaal {
 	       };
 
     require EB::Report::Journal;
+    require EB::Report::GenBase;
 
     return unless
     parse_args(\@args,
@@ -426,10 +446,12 @@ EOS
 
 sub do_balans {
     my ($self, @args) = @_;
-    require EB::Report::Balres;
     my $opts = { d_boekjaar   => $bky || $dbh->adm("bky"),
 	       };
 
+    require EB::Report::Balres;
+    require EB::Report::GenBase;
+
     return unless
     parse_args(\@args,
 	       [ 'detail=i',
@@ -462,10 +484,12 @@ EOS
 
 sub do_result {
     my ($self, @args) = @_;
-    require EB::Report::Balres;
     my $opts = { d_boekjaar   => $bky || $dbh->adm("bky"),
 	       };
 
+    require EB::Report::Balres;
+    require EB::Report::GenBase;
+
     return unless
     parse_args(\@args,
 	       [ 'detail=i',
@@ -498,11 +522,13 @@ EOS
 
 sub do_proefensaldibalans {
     my ($self, @args) = @_;
-    require EB::Report::Proof;
 
     my $opts = { d_boekjaar   => $bky || $dbh->adm("bky"),
 	       };
 
+    require EB::Report::Proof;
+    require EB::Report::GenBase;
+
     return unless
     parse_args(\@args,
 	       [ 'detail=i',
@@ -534,12 +560,14 @@ EOS
 
 sub do_grootboek {
     my ($self, @args) = @_;
-    require EB::Report::Grootboek;
 
     my $opts = { detail       => 2,
 		 d_boekjaar   => $bky || $dbh->adm("bky"),
 	       };
 
+    require EB::Report::Grootboek;
+    require EB::Report::GenBase;
+
     return unless
     parse_args(\@args,
 	       [ 'detail=i',
@@ -608,13 +636,15 @@ Toont een lijstje van beschikbare dagboeken.
 EOS
 }
 
-sub do_btwaangifte {
+# do_btwaangifte and help_btwaangifte are dynamically plugged in (or not).
+sub _do_btwaangifte {
     my ($self, @args) = @_;
     my $opts = { d_boekjaar   => $bky || $dbh->adm("bky"),
 		 close	      => 0,
 	       };
 
-    use EB::Report::BTWAangifte;
+    require EB::Report::BTWAangifte;
+    require EB::Report::GenBase;
 
     return unless
     parse_args(\@args,
@@ -622,8 +652,9 @@ sub do_btwaangifte {
 		 'periode=s'  => sub { periode_arg($opts, @_) },
 		 "definitief" => sub { $opts->{close} = 1 },
 		 EB::Report::GenBase->backend_options(EB::Report::BTWAangifte::, $opts),
+		 "noreport",
 	       ], $opts)
-      or goto &help_btwaangifte;
+      or goto &_help_btwaangifte;
 
     if ( lc($args[-1]) eq "definitief" ) {
 	$opts->{close} = 1;
@@ -636,7 +667,7 @@ sub do_btwaangifte {
     undef;
 }
 
-sub help_btwaangifte {
+sub _help_btwaangifte {
     <<EOS;
 Toont de BTW aangifte.
 
@@ -661,6 +692,9 @@ Opties:
                               met --boekjaar, en evenmin met de bovenvermelde
                               methode van periode-specificatie.
   --boekjaar=<code>           Selecteer boekjaar
+  --noreport                  Geen rapportage. Dit is enkel zinvol samen
+                              met --definitief om de afgesloten BTW periode
+                              aan te passen.
 
 Zie verder "help rapporten" voor algemene informatie over aan te maken
 rapporten.
@@ -673,6 +707,7 @@ sub do_debiteuren {
 	       };
 
     require EB::Report::Debcrd;
+    require EB::Report::GenBase;
 
     return unless
     parse_args(\@args,
@@ -706,6 +741,7 @@ sub do_crediteuren {
 	       };
 
     require EB::Report::Debcrd;
+    require EB::Report::GenBase;
 
     return unless
     parse_args(\@args,
@@ -739,6 +775,7 @@ sub do_openstaand {
 	       };
 
     require EB::Report::Open;
+    require EB::Report::GenBase;
 
     return unless
     parse_args(\@args,
@@ -821,13 +858,13 @@ sub do_relatie {
     return unless
     parse_args(\@args,
 	       [ 'dagboek=s',
-		 'btw=s',
+		 $dbh->does_btw ? 'btw=s' : (),
 	       ], $opts)
       or goto &help_relatie;
 
     warn("?"._T("Ongeldig aantal argumenten voor deze opdracht")."\n"), return if @args % 3;
 
-    use EB::Relation;
+    require EB::Relation;
 
     while ( @args ) {
 	my @a = splice(@args, 0, 3);
@@ -837,7 +874,7 @@ sub do_relatie {
 }
 
 sub help_relatie {
-    <<EOS;
+    my $ret = <<EOS;
 Aanmaken een of meer nieuwe relaties.
 
   relatie [ <opties> ] { <code> <omschrijving> <rekening> } ...
@@ -845,11 +882,15 @@ Aanmaken een of meer nieuwe relaties.
 Opties:
 
   --dagboek=<dagboek>         Selecteer dagboek voor deze relatie
+EOS
+
+    $ret .= <<EOS if $dbh->does_btw;
   --btw=<type>                BTW type: normaal, verlegd, intra, extra
 
 *** BTW type 'verlegd' wordt nog niet ondersteund ***
 *** BTW type 'intra' wordt nog niet geheel ondersteund ***
 EOS
+    $ret;
 }
 
 ################ Im/export ################
@@ -857,7 +898,9 @@ EOS
 sub do_export {
     my ($self, @args) = @_;
 
-    my $opts = {
+    my $opts = { single   => 0,
+		 explicit => 0,
+		 totals   => 1,
 	       };
 
     return unless
@@ -865,6 +908,9 @@ sub do_export {
 	       [ 'dir=s',
 		 'output=s',
 		 'boekjaar=s',
+		 'single',
+		 'explicit',
+		 'totals!',
 	       ], $opts)
       or goto &help_export;
 
@@ -878,8 +924,9 @@ sub do_export {
     }
 
     return unless argcnt(@args, 0);
+    check_open(1);
 
-    use EB::Export;
+    require EB::Export;
     EB::Export->export($opts);
 
     return;
@@ -1051,7 +1098,8 @@ sub do_verwijder {
 	       ], $opts);
     $opts->{boekjaar} = $opts->{d_boekjaar} unless defined $opts->{boekjaar};
 
-    use EB::Booking::Delete;
+    require EB::Booking::Delete;
+    require EB::Booking::Decode;
     @args = ($bsk) if $bsk && !@args;
     return _T("Gaarne een boekstuk") unless @args == 1;
     my $cmd;
@@ -1109,7 +1157,7 @@ sub do_toon {
     $opts->{trail} = !$opts->{verbose};
     $opts->{boekjaar} = $opts->{d_boekjaar} unless defined $opts->{boekjaar};
 
-    use EB::Booking::Decode;
+    require EB::Booking::Decode;
     @args = ($bsk) if $bsk && !@args;
     return _T("Gaarne een boekstuk") unless @args == 1;
     my ($id, $dbs, $err) = $dbh->bskid(shift(@args), $opts->{boekjaar});
@@ -1240,10 +1288,22 @@ sub date_arg {
 sub check_open {
     my ($self, $open) = @_;
     $open = 1 unless defined($open);
-    if ( $open && !$dbh->adm_busy ) {
+    if ( $open && !$dbh->adm_open ) {
 	die("?"._T("De administratie is nog niet geopend")."\n");
     }
-    elsif ( !$open && $dbh->adm_busy ) {
+    elsif ( !$open && $dbh->adm_open ) {
+	die("?"._T("De administratie is reeds geopend")."\n");
+    }
+    1;
+}
+
+sub check_busy {
+    my ($self, $busy) = @_;
+    $busy = 1 unless defined($busy);
+    if ( $busy && !$dbh->adm_busy ) {
+	die("?"._T("De administratie is nog niet in gebruik")."\n");
+    }
+    elsif ( !$busy && $dbh->adm_busy ) {
 	die("?"._T("De administratie is reeds in gebruik")."\n");
     }
     1;
