@@ -1,10 +1,10 @@
-# $Id: Opening.pm,v 1.19 2006/02/04 12:00:15 jv Exp $
+# $Id: Opening.pm,v 1.23 2006/03/05 20:57:49 jv Exp $
 
 # Author          : Johan Vromans
 # Created On      : Tue Aug 30 09:49:11 2005
 # Last Modified By: Johan Vromans
-# Last Modified On: Sat Feb  4 12:38:28 2006
-# Update Count    : 167
+# Last Modified On: Sun Mar  5 21:37:50 2006
+# Update Count    : 198
 # Status          : Unknown, Use with caution!
 
 package main;
@@ -81,9 +81,10 @@ sub set_boekjaarcode {
 sub set_balanstotaal {
     return shellhelp() unless @_ == 2;
     my ($self, $amt) = @_;
-    return __x("Ongeldig bedrag: {amount}", amount => $amt)."\n" unless defined($amt = amount($amt));
+    my $anew;
+    return __x("Ongeldig bedrag: {amount}", amount => $amt)."\n" unless defined($anew = amount($amt));
     $self->check_open(0);
-    $self->{o}->{balanstotaal} = $amt;
+    $self->{o}->{balanstotaal} = $anew;
     "";
 }
 
@@ -106,9 +107,10 @@ sub set_balans {
     else {
 	$debcrd = $rr->[1];
     }
-    return __x("Ongeldig bedrag: {amount}", amount => $amt)."\n" unless defined($amt = amount($amt));
+    my $anew;
+    return __x("Ongeldig bedrag: {amount}", amount => $amt)."\n" unless defined($anew = amount($amt));
     $self->check_open(0);
-    push(@{$self->{o}->{balans}}, [$acct, $debcrd ? $amt : -$amt]);
+    push(@{$self->{o}->{balans}}, [$acct, $debcrd ? $anew : -$anew]);
     "";
 }
 
@@ -156,10 +158,11 @@ sub set_relatie {
     return __x("Ongeldige relatie: {rel}", rel => $code)."\n"
       if $type  ^ $debcrd;
 
-    return __x("Ongeldig bedrag: {amount}", amount => $amt)."\n" unless defined($amt = amount($amt));
+    my $anew;
+    return __x("Ongeldig bedrag: {amount}", amount => $amt)."\n" unless defined($anew = amount($amt));
 
     $self->check_open(0);
-    push(@{$self->{o}->{relatie}}, [$bky, $nr, $date, $desc, $type, $code, $amt]);
+    push(@{$self->{o}->{relatie}}, [$bky, $nr, $date, $desc, $type, $code, $anew]);
     "";
 }
 
@@ -179,6 +182,7 @@ sub open {
       unless $o->{naam};
     $fail++, warn(_T("De begindatum is nog niet opgegeven")."\n")
       unless $o->{begindatum};
+    my $does_btw = $dbh->does_btw;
 
     my $gbj;
     unless ( $gbj = defined($o->{boekjaarcode}) ) {
@@ -195,17 +199,31 @@ sub open {
     }
 
     $fail++, warn(_T("De BTW periode is nog niet opgegeven")."\n")
-      unless $o->{btwperiode};
+      if $does_btw && !$o->{btwperiode};
     if ( ($o->{balans} || $o->{relatie}) && !defined($o->{balanstotaal}) ) {
 	$fail++;
 	warn(_T("Het totaalbedrag van de openingsbalans is nog niet opgegeven")."\n");
 
     }
+
+    # Generalise for multiple deb/crd accounts.
+    my %adeb;
+    my %acrd;
+    my $sth = $dbh->sql_exec("SELECT dbk_acc_id FROM Dagboeken".
+			     " WHERE dbk_type = ?", DBKTYPE_INKOOP);
+    while ( my $rr = $sth->fetch ) {
+	$acrd{0+$rr->[0]} = 0;
+    }
+    $sth = $dbh->sql_exec("SELECT dbk_acc_id FROM Dagboeken".
+			  " WHERE dbk_type = ?", DBKTYPE_VERKOOP);
+    while ( my $rr = $sth->fetch ) {
+	$adeb{0+$rr->[0]} = 0;
+    }
+
     if ( $o->{balanstotaal} ) {
-	my $adeb = $dbh->std_acc("deb");
-	my $acrd = $dbh->std_acc("crd");
-	my $rdeb;
-	my $rcrd;
+	my $adeb;
+	my $acrd;
+	my $need_rel = 0;
 	if ( !$o->{balans} ) {
 	    $fail++;
 	    warn(_T("De openingsbalans is nog niet opgegeven")."\n");
@@ -221,8 +239,8 @@ sub open {
 		else {
 		    $credit -= $amt;
 		}
-		$rdeb = $amt if $acct == $adeb;
-		$rcrd = $amt if $acct == $acrd;
+		$need_rel++, $adeb{$acct} = $amt if defined($adeb{$acct});
+		$need_rel++, $acrd{$acct} = $amt if defined($acrd{$acct});
 	    }
 	    $fail++, warn(_T("De openingsbalans is niet in balans!")."\n".
 			  __x("Totaal = {total}, residu debet = {rdeb}, residu credit = {rcrd}",
@@ -230,21 +248,46 @@ sub open {
 			      rdeb => numfmt($debet),
 			      rcrd => numfmt(-$credit))."\n")
 	      if $debet || $credit;
-	    $fail++, warn(_T("De openstaande debiteuren zijn nog niet opgegeven")."\n")
-	      if defined($rdeb) && !$o->{relatie};
-	    $fail++, warn(_T("De openstaande crediteuren zijn nog niet opgegeven")."\n")
-	      if defined($rcrd) && !$o->{relatie};
-	    $fail++, warn(_T("Er zijn openstaande posten opgegeven, maar geen corresponderende balansposten")."\n")
-	      if $o->{relatie} && !(defined($rdeb) || defined($rcrd));
 
+	    # Helpful hints...
+	    $fail++, warn(_T("Er zijn geen openstaande posten opgegeven")."\n")
+	      if !$o->{relatie} && $need_rel;
+	    $fail++, warn(_T("Er zijn openstaande posten opgegeven, maar geen corresponderende balansposten")."\n")
+	      if $o->{relatie} && !$need_rel;
+
+	    # Process relations.
 	    foreach my $r ( @{$o->{relatie}} ) {
 		my ($bky, $nr, $date, $desc, $debcrd, $code, $amt) = @$r;
+
 		if ( $debcrd ) {
-		    $rdeb -= $amt;
+		    $adeb ||= $dbh->std_acc("deb", $adeb);
+		    unless ( defined $adeb ) {
+			$adeb = (keys(%adeb))[0] if scalar(keys(%adeb)) == 1;
+		    }
+		    unless ( defined $adeb ) {
+			warn(_T("Deze administratie kent geen koppeling voor verkoopboekingen")."\n");
+			$fail++;
+			$adeb = 0;
+			next;
+		    }
+		    next unless $adeb;
+		    $adeb{$adeb} -= $amt;
 		}
 		else {
-		    $rcrd += $amt;
+		    $acrd ||= $dbh->std_acc("crd", $acrd);
+		    unless ( defined $acrd ) {
+			$acrd = (keys(%acrd))[0] if scalar(keys(%acrd)) == 1;
+		    }
+		    unless ( defined $acrd ) {
+			warn(_T("Deze administratie kent geen koppeling voor inkoopboekingen")."\n");
+			$fail++;
+			$acrd = 0; # defined
+			next;
+		    }
+		    next unless $acrd;
+		    $acrd{$acrd} += $amt;
 		}
+
 		if ( defined($bky) ) {
 		    my $sth = $dbh->sql_exec("SELECT bky_begin, bky_end".
 					     " FROM Boekjaren".
@@ -273,22 +316,36 @@ sub open {
 		    $bky = BKY_PREVIOUS;
 		}
 	    }
-	    $fail++, warn(($rdeb >= 0 ?
-			   __x("Er is {amt} te weinig aan openstaande {dc} opgegeven",
-			       amt => numfmt($rdeb),
-			       dc => lc(_T("Debiteuren"))) :
-			   __x("Er is {amt} te veel aan openstaande {dc} opgegeven",
-			       amt => numfmt(-$rdeb),
-			       dc => lc(_T("Debiteuren"))))."\n")
-	      if $rdeb;
-	    $fail++, warn(($rcrd >= 0 ?
-			   __x("Er is {amt} te weinig aan openstaande {dc} opgegeven",
-			       amt => numfmt($rcrd),
-			       dc => lc(_T("Crediteuren"))) :
-			   __x("Er is {amt} te veel aan openstaande {dc} opgegeven",
-			       amt => numfmt(-$rcrd),
-			       dc => lc(_T("Crediteuren"))))."\n")
-	      if $rcrd;
+
+	    foreach my $adeb ( keys(%adeb) ) {
+		next unless $adeb{$adeb};
+		$fail++;
+		if ( $adeb{$adeb} >= 0 ) {
+		    warn(__x("Er is {amt} te weinig aan openstaande {dc} (rekening {acct}) opgegeven",
+			     amt => numfmt($adeb{$adeb}), acct => $adeb,
+			     dc => lc(_T("Debiteuren")))."\n");
+		}
+		else {
+		    warn(__x("Er is {amt} te veel aan openstaande {dc} (rekening {acct}) opgegeven",
+			     amt => numfmt(-$adeb{$adeb}), acct => $adeb,
+			     dc => lc(_T("Debiteuren")))."\n");
+		}
+	    }
+
+	    foreach my $acrd ( keys(%acrd) ) {
+		next unless $acrd{$acrd};
+		$fail++;
+		if ( $acrd{$acrd} >= 0 ) {
+		    warn(__x("Er is {amt} te veel aan openstaande {dc} (rekening {acct}) opgegeven",
+			     amt => numfmt($acrd{$acrd}), acct => $acrd,
+			     dc => lc(_T("Crediteuren")))."\n");
+		}
+		else {
+		    warn(__x("Er is {amt} te weinig aan openstaande {dc} (rekening {acct}) opgegeven",
+			     amt => numfmt(-$acrd{$acrd}), acct => $acrd,
+			     dc => lc(_T("Crediteuren")))."\n");
+		}
+	    }
 	}
     }
     return _T("DE OPENING IS NIET UITGEVOERD!")."\n" if $fail;
@@ -299,10 +356,11 @@ sub open {
 		     [qw(bky_code bky_name bky_begin bky_end bky_btwperiod bky_opened)],
 		     $o->{boekjaarcode}, $o->{naam},
 		     $o->{begindatum} . "-01-01", $o->{begindatum} . "-12-31", # TODO
-		     $o->{btwperiode}, $now);
+		     $o->{btwperiode}||0, $now);
     $dbh->sql_exec("UPDATE Metadata".
 		   " SET adm_bky = ?, adm_btwbegin = ?",
-		   $o->{boekjaarcode}, $o->{begindatum} . "-01-01");
+		   $o->{boekjaarcode},
+		   $does_btw ? $o->{begindatum} . "-01-01" : undef);
     $dbh->sql_exec("UPDATE Boekjaren".
 		   " SET bky_closed = ?, bky_end = ?".
 		   " WHERE bky_code = ?",
@@ -350,10 +408,10 @@ sub open {
 			     $nr, $desc, $dagboek, $date, $bky, $amt, $amt);
 	    $dbh->sql_insert("Boekstukregels",
 			     [qw(bsr_nr bsr_date bsr_bsk_id bsr_desc bsr_rel_code bsr_amount
-				 bsr_type)],
+				 bsr_type bsr_btw_class)],
 			     1, $date,
 			     $dbh->get_sequence("boekstukken_bsk_id_seq", "noincr"),
-			     $desc, $code, 0-$amt, 9);
+			     $desc, $code, 0-$amt, 9, 0);
 	}
 #	my $highest = $dbh->get_sequence("bsk_nr_0_seq") + 1;
 #	$dbh->set_sequence("bsk_nr_${dbk_inkoop}_seq", $highest)
@@ -396,7 +454,7 @@ sub reopen {
     $o->{begindatum} = $y;
 
     warn(_T("Er is geen nieuwe BTW periode opgegeven, deze blijft ongewijzigd")."\n")
-      unless $o->{btwperiode};
+      if $dbh->does_btw && !$o->{btwperiode};
 
     if ( !defined($o->{boekjaarcode}) ) {
 	warn(__x("Er is geen boekjaarcode opgegeven, de waarde {val} wordt gebruikt",
