@@ -1,10 +1,10 @@
-my $RCS_Id = '$Id: Schema.pm,v 1.31 2006/02/07 11:44:05 jv Exp $ ';
+my $RCS_Id = '$Id: Schema.pm,v 1.39 2006/03/05 20:58:08 jv Exp $ ';
 
 # Author          : Johan Vromans
 # Created On      : Sun Aug 14 18:10:49 2005
 # Last Modified By: Johan Vromans
-# Last Modified On: Tue Feb  7 12:20:08 2006
-# Update Count    : 478
+# Last Modified On: Sun Mar  5 21:05:03 2006
+# Update Count    : 570
 # Status          : Unknown, Use with caution!
 
 ################ Common stuff ################
@@ -146,8 +146,8 @@ sub scan_dagboeken {
     error(__x("Dagboek {id}: het :type ontbreekt", id => $id)."\n") unless defined($type);
     error(__x("Dagboek {id}: het :rekening nummer ontbreekt", id => $id)."\n")
       if ( $type == DBKTYPE_KAS || $type == DBKTYPE_BANK ) and !$type;
-    error(__x("Dagboek {id}: rekeningnummer enkel toegestaan voor Kas en Bankboeken", id => $id)."\n")
-      if $rek && !($type == DBKTYPE_KAS || $type == DBKTYPE_BANK || $type == DBKTYPE_MEMORIAAL);
+#    error(__x("Dagboek {id}: rekeningnummer enkel toegestaan voor Kas en Bankboeken", id => $id)."\n")
+#      if $rek && !($type == DBKTYPE_KAS || $type == DBKTYPE_BANK || $type == DBKTYPE_MEMORIAAL);
 
     $dbk{$id} = $dbkid;
     $dbk[$dbkid] = [ $id, $desc, $type, $rek||undef ];
@@ -181,8 +181,10 @@ sub scan_btw {
 	elsif ( $extra =~ m/^tariefgroep=laag$/i ) {
 	    $groep = BTWTARIEF_LAAG;
 	}
-	elsif ( $extra =~ m/^tariefgroep=geen$/i ) {
-	    $groep = BTWTARIEF_GEEN;
+	elsif ( $extra =~ m/^tariefgroep=(nul|geen)$/i ) {
+	    $groep = BTWTARIEF_NUL;
+	    warn("!"._T("Gelieve BTW tariefgroep \"Geen\" te vervangen door \"Nul\"")."\n")
+	      if lc($1) eq "geen";
 	}
 	elsif ( $extra =~ m/^incl(?:usief)?$/i ) {
 	    $incl = 1;
@@ -198,12 +200,12 @@ sub scan_btw {
 
     error(__x("BTW tarief {id}: geen percentage en de tariefgroep is niet \"{none}\"",
 	      id => $id, none => _T("geen"))."\n")
-      unless defined($perc) || $groep == BTWTARIEF_GEEN;
+      unless defined($perc) || $groep == BTWTARIEF_NUL;
 
     $btw[$id] = [ $id, $desc, $groep, $perc, $incl ];
 
-    if ( $groep == BTWTARIEF_GEEN && !defined($btwmap{g}) ) {
-	$btwmap{g} = $id;
+    if ( $groep == BTWTARIEF_NUL && !defined($btwmap{n}) ) {
+	$btwmap{n} = $id;
     }
     elsif ( $incl ) {
 	if ( $groep == BTWTARIEF_HOOG && !defined($btwmap{h}) ) {
@@ -213,6 +215,14 @@ sub scan_btw {
 	    $btwmap{l} = $id;
 	}
     }
+    else {
+	if ( $groep == BTWTARIEF_HOOG && !defined($btwmap{"h-"}) ) {
+	    $btwmap{"h-"} = $id;
+	}
+	elsif ( $groep == BTWTARIEF_LAAG && !defined($btwmap{"l-"}) ) {
+	    $btwmap{"l-"} = $id;
+	}
+    }
     $btwmap{$id} = $id;
     1;
 }
@@ -233,28 +243,56 @@ sub scan_balres {
 	error(__x("Dubbel: rekening {acct}", acct => $1)."\n") if exists($acc{$id});
 	error(__x("Rekening {id} heeft geen verdichting", id => $id)."\n") unless defined($cvdi);
 	my $debcrd;
-	my $kstomz = 1;
-	if ( ($balres ? $flags =~ /^[dc]$/i : $flags =~ /^[ko]$/i)
+	my $kstomz;
+	if ( ($balres ? $flags =~ /^[dc]$/i : $flags =~ /^[kon]$/i)
 	     ||
 	     $flags =~ /^[dc][ko]$/i ) {
 	    $debcrd = $flags =~ /d/i;
-	    $kstomz = $flags =~ /k/i;
+	    $kstomz = $flags =~ /k/i if $flags =~ /[ko]/i;
 	}
 	else {
 	    error(__x("Rekening {id}: onherkenbare vlaggetjes {flags}",
 		      id => $id, flags => $flags)."\n");
 	}
 
-	my $btw = 'g';
+	my $btw_type = 'n';
+	my $btw_ko;
 	my $extra;
+
 	while ( $desc =~ /^(.+?)\s+:([^\s:]+)\s*$/ ) {
 	    $desc = $1;
 	    $extra = $2;
-	    if ( $extra =~ m/^btw=(hoog|laag)$/i ) {
-		$btw = lc(substr($1,0,1));
-	    }
-	    elsif ( $extra =~ m/^btw=(\d+)$/i ) {
-		$btw = $1;
+	    if ( $extra =~ m/^btw=(.+)$/i ) {
+		my $spec = $1;
+		my @spec = split(/,/, lc($spec));
+
+		my $btw_inex = 1;
+
+		foreach ( @spec ) {
+		    if ( $balres && /^(kosten|omzet)$/ ) {
+			$btw_ko = substr($1, 0, 1) eq "k";
+		    }
+		    elsif ( /^(hoog|laag|nul)$/ ) {
+			$btw_type = substr($1, 0, 1);
+		    }
+		    elsif ( /^\d+$/ ) {
+			$btw_type = $_;
+		    }
+		    elsif ( $_ eq "geen" ) {
+			$btw_type = 0;
+			$kstomz = $btw_ko = undef;
+		    }
+		    elsif ( /^(incl|excl)(usief)?$/ ) {
+			$btw_inex = substr($1, 1, 1) eq 'i';
+		    }
+		    else {
+			error(__x("Foutieve BTW specificatie: {spec}",
+				  spec => $spec)."\n");
+			last;
+		    }
+		}
+
+		$btw_type .= "-" unless $btw_inex;
 	    }
 	    elsif ( $extra =~ m/koppeling=(\S+)/i ) {
 		error(__x("Rekening {id}: onbekende koppeling \"{std}\"",
@@ -266,8 +304,20 @@ sub scan_balres {
 		$std{$1} = $id;
 	    }
 	}
+	if ( $btw_type ne 'n' ) {
+	    error(__x("Rekening {id}: BTW koppeling '{ko}' met een {acc} is niet toegestaan",
+		      id => $id, ko => qw(omzet kosten)[$btw_ko],
+		      acc => qw(omzetrekening kostenrekening)[$kstomz])."\n")
+	      if !$balres && defined($kstomz) && defined($btw_ko) && $btw_ko != $kstomz;
+	    error(__x("Rekening {id}: BTW koppeling met neutrale resultaatrekening is niet toegestaan",
+		      id => $id)."\n") unless defined($kstomz) || defined($btw_ko);
+	    error(__x("Rekening {id}: BTW koppeling met een balansrekening vereist kosten/omzet specificatie",
+		      id => $id)."\n")
+	      if $balres && !defined($btw_ko);
+	}
 	$desc =~ s/\s+$//;
-	$acc{$id} = [ $desc, $cvdi, $balres, $debcrd, $kstomz, $btw ];
+	$kstomz = $btw_ko unless defined($kstomz);
+	$acc{$id} = [ $desc, $cvdi, $balres, $debcrd, $kstomz, $btw_type ];
     }
     else {
 	0;
@@ -335,17 +385,53 @@ sub load_schema {
 		     " \"Dagboeken\" of \"BTW Tarieven\"")."\n");
     }
 
+    # Bekijk alle dagboeken om te zien of er inkoop/verkoop dagboeken
+    # zijn die een tegenrekening nodig hebben. In dat geval moet de
+    # betreffende koppeling in het schema gemaakt zijn.
+    my ($need_deb, $need_crd) = (0,0);
+    foreach ( @dbk ) {
+	next unless defined($_); # sparse
+	my ($id, $desc, $type, $rek) = @$_;
+	next if defined($rek);
+	if ( $type == DBKTYPE_INKOOP ) {
+	    $need_crd++;
+	    $_->[3] = $std{"crd"};
+	    #### Verify that it's a C acct.
+	}
+	elsif ( $type == DBKTYPE_VERKOOP ) {
+	    $need_deb++;
+	    $_->[3] = $std{"deb"};
+	    #### Verify that it's a D acct.
+	}
+	elsif ( $type != DBKTYPE_MEMORIAAL ) {
+	    error(__x("Dagboek {id} heeft geen tegenrekening", id => $id)."\n");
+	    $fail++;
+	}
+    }
+    # Verwijder onnodige koppelingen.
+    delete($std{crd}) unless $need_crd;
+    delete($std{deb}) unless $need_deb;
+
+    my %mapbtw = ( n => "Nul", h => "Hoog", "l" => "Laag" );
+    if ( @btw ) {
+	foreach ( keys(%mapbtw) ) {
+	    next if defined($btwmap{$_});
+	    error(__x("Geen BTW tarief gevonden met tariefgroep {gr}, inclusief",
+		      gr => $mapbtw{$_})."\n");
+	}
+    }
+    else {
+	for ( qw(ih il vh vl ok) ) {
+	    delete($std{"btw_$_"}) unless $std{"btw_$_"};
+	}
+	$btwmap{n} = undef;
+	$btw[0] = [ 0, "BTW Nul", BTWTARIEF_NUL, 0, 0 ];
+    }
     while ( my($k,$v) = each(%std) ) {
 	next if $v;
 	error(__x("Geen koppeling gevonden voor \"{std}\"", std => $k)."\n");
     }
 
-    my %mapbtw = ( g => "Geen", h => "Hoog", "l" => "Laag" );
-    foreach ( keys(%mapbtw) ) {
-	next if defined($btwmap{$_});
-	error(__x("Geen BTW tarief gevonden met tariefgroep {gr}, inclusief",
-		  gr => $mapbtw{$_})."\n");
-    }
     die("?"._T("FOUTEN GEVONDEN, VERWERKING AFGEBROKEN")."\n") if $fail;
 
     if ( $sql ) {
@@ -409,7 +495,7 @@ ESQL
     for ( my $i = 0; $i < @hvdi; $i++ ) {
 	next unless exists $hvdi[$i];
 	my $v = $hvdi[$i];
-	$out .= _tsv($i, $v->[0], _tf($v->[1]), "\\N", "\\N");
+	$out .= _tsv($i, $v->[0], _tf($v->[1]), _tfn(undef), "\\N");
     }
     $out .= "\\.\n";
 
@@ -422,7 +508,7 @@ ESQL
     for ( my $i = 0; $i < @vdi; $i++ ) {
 	next unless exists $vdi[$i];
 	my $v = $vdi[$i];
-	$out .= _tsv($i, $v->[0], _tf($v->[1]), "\\N", $v->[2]);
+	$out .= _tsv($i, $v->[0], _tf($v->[1]), _tfn(undef), $v->[2]);
     }
     $out . "\\.\n";
 }
@@ -439,12 +525,12 @@ ESQL
     for my $i ( sort { $a <=> $b } keys(%acc) ) {
 	my $g = $acc{$i};
 	croak(__x("Geen BTW tariefgroep voor code {code}",
-		  code => $g->[5])) unless defined $btwmap{$g->[5]};
+		  code => $g->[5])) unless exists $btwmap{$g->[5]};
 	$out .= _tsv($i, $g->[0], $g->[1],
 		     _tf($g->[2]),
 		     _tf($g->[3]),
-		     _tf($g->[4]),
-		     $btwmap{$g->[5]},
+		     _tfn($g->[4]),
+		     defined($btwmap{$g->[5]}) ? $btwmap{$g->[5]} : "\\N",
 		     0, 0);
     }
     $out . "\\.\n";
@@ -469,7 +555,7 @@ ESQL
 
     foreach ( @btw ) {
 	next unless defined;
-	if ( $_->[2] == BTWTARIEF_GEEN ) {
+	if ( $_->[2] == BTWTARIEF_NUL ) {
 	    $_->[3] = 0;
 	    $_->[4] = "\\N";
 	}
@@ -489,8 +575,8 @@ ESQL
 
     foreach ( @dbk ) {
 	next unless defined;
-	$_->[3] = $std{deb} if $_->[2] == DBKTYPE_VERKOOP;
-	$_->[3] = $std{crd} if $_->[2] == DBKTYPE_INKOOP;
+	$_->[3] ||= $std{deb} if $_->[2] == DBKTYPE_VERKOOP;
+	$_->[3] ||= $std{crd} if $_->[2] == DBKTYPE_INKOOP;
 	$out .= join("\t",
 		     map { defined($_) ? $_ : "\\N" } @$_).
 		       "\n";
@@ -579,12 +665,13 @@ print {$fh}  <<EOD;
 #
 # De omschrijving van de grootboekrekeningen wordt voorafgegaan door
 # een vlaggetje, een letter die resp. Debet/Credit (voor
-# balansrekeningen) en Kosten/Omzet (voor resultaatrekeningen)
+# balansrekeningen) en Kosten/Omzet/Neutraal (voor resultaatrekeningen)
 # aangeeft. De omschrijving wordt indien nodig gevolgd door extra
 # informatie. Voor grootboekrekeningen kan op deze wijze de BTW
 # tariefstelling worden aangegeven die op deze rekening van toepassing
 # is:
 #
+#   :btw=nul
 #   :btw=hoog
 #   :btw=laag
 #
@@ -592,8 +679,8 @@ print {$fh}  <<EOD;
 # (speciale betekenis) heeft met :koppeling=xxx. De volgende koppelingen
 # zijn mogelijk:
 #
-#   crd		de tegenrekening (Crediteuren) voor inkoopboekingen
-#   deb		de tegenrekening (Debiteuren) voor verkoopboekingen
+#   crd		de standaard tegenrekening (Crediteuren) voor inkoopboekingen
+#   deb		de standaard tegenrekening (Debiteuren) voor verkoopboekingen
 #   btw_ih	de rekening voor BTW boekingen voor inkopen, hoog tarief
 #   btw_il	idem, laag tarief
 #   btw_vh	idem, verkopen, hoog tarief
@@ -601,25 +688,16 @@ print {$fh}  <<EOD;
 #   btw_ok	rekening voor de betaalde BTW
 #   winst	rekening waarop de winst wordt geboekt
 #
-# Al deze koppelingen moeten nmaal in het rekeningschema voorkomen.
-
+# De koppeling winst is verplicht en moet altijd in een administratie
+# voorkomen in verband met de jaarafsluiting.
+# De koppelingen voor BTW moeten worden opgegeven indien BTW
+# van toepassing is op de administratie.
+# De koppelingen voor Crediteuren en Debiteuren moeten worden
+# opgegeven indien er inkoop dan wel verkoopdagboeken zijn die gebruik
+# maken van de standaardwaarden (dus zelf geen tegenrekening hebben
+# opgegeven).
 EOD
 
-=begin not_now
-
-# De classificatie Kosten/Omzet voor resultaatrekeningen wordt onder
-# meer gebruikt om te bepalen hoe BTW moet worden geboekt. Als om
-# welke reden dan ook de classificatie niet betrouwbaar moet worden
-# geacht, kan de aanduiding "Kosten-Omzet NOK" worden gebruikt om aan
-# te geven dat EekBoek deze niet mag gebruiken. Dit kan echter leiden
-# tot problemen met BTW boekingen!
-
-EOD
-
-    print {$fh} ($dbh->adm_ko ? "# " : "", "Kosten-Omzet NOK\n");
-
-=cut
-
 $max_hvd = $dbh->do("SELECT MAX(vdi_id) FROM Verdichtingen WHERE vdi_struct IS NULL")->[0];
 $max_vrd = $dbh->do("SELECT MAX(vdi_id) FROM Verdichtingen WHERE NOT vdi_struct IS NULL")->[0];
 
@@ -656,24 +734,26 @@ print {$fh}  <<EOD;
 # dagboeken worden aangemaakt.
 # In de eerste kolom wordt de korte naam (code) voor het dagboek
 # opgegeven. Verder moet voor elk dagboek worden opgegeven van welk
-# type het is. Voor rekeningen van het type Kas en Bank moet bovendien
-# een tegenrekening worden opgegeven.
+# type het is. Voor dagboeken van het type Kas en Bank moet een
+# tegenrekening worden opgegeven, voor de overige dagboeken mag een
+# tegenrekening worden opgegeven.
 EOD
 
     dump_dbk($fh);			# Dagboeken
 
-print {$fh}  <<EOD;
+    if ( $dbh->does_btw ) {
+	print {$fh}  <<EOD;
 
 # BTW TARIEVEN
 #
-# Er zijn drie tariefgroepen: "hoog", "laag" en "geen". De tariefgroep
+# Er zijn drie tariefgroepen: "hoog", "laag" en "nul". De tariefgroep
 # bepaalt het rekeningnummer waarop de betreffende boeking plaatsvindt.
 # Binnen elke tariefgroep zijn meerdere tarieven mogelijk, hoewel dit
 # in de praktijk niet snel zal voorkomen.
 # In de eerste kolom wordt de (numerieke) code voor dit tarief
 # opgegeven. Deze kan o.m. worden gebruikt om expliciet een BTW tarief
 # op te geven bij het boeken. Voor elk tarief (behalve die van groep
-# "geen") moet het percentage worden opgegeven. Met de aanduiding
+# "nul") moet het percentage worden opgegeven. Met de aanduiding
 # :exclusief kan worden opgegeven dat boekingen op rekeningen met deze
 # tariefgroep standaard het bedrag exclusief BTW aangeven.
 #
@@ -683,8 +763,8 @@ print {$fh}  <<EOD;
 # hebben voor de reeds in scripts vastgelegde boekingen.
 EOD
 
-    dump_btw($fh);			# BTW tarieven
-
+	dump_btw($fh);			# BTW tarieven
+    }
 print {$fh}  <<EOD;
 
 # Einde EekBoek schema
@@ -721,7 +801,8 @@ sub dump_acc {
 				     " acc_btw, btw_tariefgroep, btw_incl".
 				     " FROM Accounts, BTWTabel ".
 				     " WHERE acc_struct = ?".
-				     " AND btw_id = acc_btw".
+				     " AND (btw_id = acc_btw".
+				     " OR btw_id = 0 AND acc_btw IS NULL)".
 				     " ORDER BY acc_id", $id);
 	    while ( my $rr = $sth->fetchrow_arrayref ) {
 		my ($id, $desc, $acc_balres, $acc_debcrd, $acc_kstomz, $btw_id, $btw, $btwincl) = @$rr;
@@ -730,18 +811,36 @@ sub dump_acc {
 		    $flags .= $acc_debcrd ? "D" : "C";
 		}
 		else {
-		    $flags .= $acc_kstomz ? "K" : "O";
+		    $flags .= defined($acc_kstomz)
+		      ? ($acc_kstomz ? "K" : "O")
+			: "N";
 		}
 		my $extra = "";
-		if ( $btw == BTWTARIEF_HOOG && $btwincl ) {
+		if ( $btw == BTWTARIEF_HOOG ) {
 		    $extra .= " :btw=hoog";
+		    $extra .= ",excl" unless $btwincl;
+		    if ( $balres ) {
+			$extra .= ",kosten" if $acc_kstomz;
+			$extra .= ",omzet"  if !$acc_kstomz;
+		    }
 		}
-		elsif ( $btw == BTWTARIEF_LAAG && $btwincl ) {
+		elsif ( $btw == BTWTARIEF_LAAG ) {
 		    $extra .= " :btw=laag";
+		    $extra .= ",excl" unless $btwincl;
+		    if ( $balres ) {
+			$extra .= ",kosten" if $acc_kstomz;
+			$extra .= ",omzet"  if !$acc_kstomz;
+		    }
 		}
-		elsif ( $btw != BTWTARIEF_GEEN ) {
+		elsif ( $btw != BTWTARIEF_NUL ) {
 		    $extra .= " :btw=$btw_id";
 		}
+		else {
+		    if ( $balres && defined($acc_kstomz) ) {
+			$extra .= " :btw=kosten" if $acc_kstomz;
+			$extra .= " :btw=omzet"  if !$acc_kstomz;
+		    }
+		}
 		$extra .= " :koppeling=".$kopp{$id} if exists($kopp{$id});
 		$desc =~ s/^\s+//;
 		$desc =~ s/\s+$//;
@@ -769,7 +868,7 @@ sub dump_btw {
 	my ($id, $desc, $perc, $btg, $incl) = @$rr;
 	my $extra = "";
 	$extra .= " :tariefgroep=" . lc(BTWTARIEVEN->[$btg]);
-	if ( $btg != BTWTARIEF_GEEN ) {
+	if ( $btg != BTWTARIEF_NUL ) {
 	    $extra .= " :perc=".btwfmt($perc);
 	    $extra .= " :" . qw(exclusief inclusief)[$incl] unless $incl;
 	}
@@ -788,7 +887,8 @@ sub dump_dbk {
 			     " ORDER BY dbk_id");
     while ( my $rr = $sth->fetchrow_arrayref ) {
 	my ($id, $desc, $type, $acc_id) = @$rr;
-	$acc_id = 0 if $type == DBKTYPE_INKOOP || $type == DBKTYPE_VERKOOP;
+	$acc_id = 0 if $type == DBKTYPE_INKOOP  && $dbh->std_acc("crd", 0) == $acc_id;
+	$acc_id = 0 if $type == DBKTYPE_VERKOOP && $dbh->std_acc("deb", 0) == $acc_id;
 	my $t = sprintf("  %-4s  %-20s  :type=%-10s %s",
 			$id, $desc, lc(DBKTYPES->[$type]),
 			($acc_id ? ":rekening=$acc_id" : ""));
